import { jsx as _jsx } from "react/jsx-runtime";
import { useState, useContext, createContext, useEffect, } from 'react';
const modifiers = ['ctrl', 'meta', 'shift', 'alt'];
function parseHandlers(handlers) {
    return Object.keys(handlers).map(key => {
        const output = {
            name: key,
            handler: handlers[key],
            key: '',
        };
        const tokens = key.split('+');
        tokens.forEach((token, index) => {
            if (index === tokens.length - 1) {
                output.key = token;
                return;
            }
            const lowerCaseToken = token.toLowerCase();
            modifiers.forEach((mod) => {
                if (lowerCaseToken === mod) {
                    output[mod] = true;
                }
            });
        });
        return output;
    });
}
const KeyboardContext = createContext({
    focusedElementID: '',
    setFocusedElementID: () => {
        return;
    },
});
export default function useKeyboard({ id = '', handlers, global, autoFocus, }) {
    const { focusedElementID, setFocusedElementID } = useContext(KeyboardContext);
    const hasFocus = global || focusedElementID === id;
    useEffect(() => {
        if (autoFocus) {
            focus();
        }
    }, [autoFocus]);
    useEffect(() => {
        if (!hasFocus) {
            return;
        }
        const parsed = parseHandlers(handlers);
        function onKeyDown(event) {
            if (!hasFocus) {
                return;
            }
            const found = parsed.find(x => x.key === event.key);
            if (!found) {
                return;
            }
            for (const mod of modifiers) {
                if (found[mod] && !event[`${mod}Key`]) {
                    return;
                }
            }
            found.handler(event);
        }
        document.addEventListener('keydown', onKeyDown);
        return () => document.removeEventListener('keydown', onKeyDown);
    }, [handlers, hasFocus]);
    function focus() {
        setFocusedElementID(id);
    }
    return { focus, hasFocus };
}
export function KeyboardProvider({ children, }) {
    const [focusedElementID, setFocusedElementID] = useState('');
    return (_jsx(KeyboardContext.Provider, Object.assign({ value: { focusedElementID, setFocusedElementID } }, { children: children })));
}
